//+-------------------------------------------------------------------------------------------------------------------------------------------------+
//|                                                                                                                             RSI_vs_BB_vs_MA.mq4 |
//+-------------------------------------------------------------------------------------------------------------------------------------------------+

/*------------------------------------------------------------------------------------------------------------------------------- specification ---//
 RSI+BB+MA.
 RSI  .
Bollinger Bands    RSI (  first indicator`s data).
MA     RSI (  first indicator`s data).

 "" :
 RSI -  ,   , ,  (30  70,  ). 
 BB - , ,  .
  - ,  ,  .

     ,  "".

          (  -  ,   -  ).
 ,    ,      .      
(  ) .

,    :
1.    ( ):
- RSI     (30  )
- RSI   Bollinger Bands     
- RSI    
-  ,      . 
2.    ( ):
- RSI     (70  )
- RSI   Bollinger Bands     
- RSI    
-  ,      .

          .
          . 
       . 
//-------------------------------------------------------------------------------------------------------------------------------------------------*/
#property indicator_separate_window
#property indicator_buffers 6
#property indicator_color1 Red
#property indicator_color2 DodgerBlue
#property indicator_color3 DodgerBlue
#property indicator_color4 Yellow
#property indicator_color5 Blue
#property indicator_color6 Red
#property indicator_minimum 0
#property indicator_maximum 100
//========================================================================================================================== external variables ===//
extern string _1_="___ RSI ___";
extern int rsi_period=8, 
           rsi_price=0;
extern double up_level=70.0,
              dn_level=30.0;
extern string _2_="___ BB ___";
extern int bb_period=20;
extern double bb_dev=2.0;
extern string _3_="___ MA ___"; 
extern    int ma_period=21,
              ma_method=0;
extern string _4_="___ Arrows ___";
extern bool use_arrows=false;
//============================================================================================================================ global variables ===//
double Buff0[], // RSI
       Buff1[], // BB UP
       Buff2[], // BB DOWN
       Buff3[], // MA
       Buff4[], // signal UP
       Buff5[]; // signal DOWN
//================================================================================================ The initialization function of the indicator ===//
int init()
  {
   int period=rsi_period;  if(period<bb_period) period=bb_period;  if(period<ma_period) period=ma_period;

   SetIndexStyle(0,DRAW_LINE); SetIndexBuffer(0,Buff0); SetIndexDrawBegin(0,period); SetIndexLabel(0,"RSI");
   SetIndexStyle(1,DRAW_LINE); SetIndexBuffer(1,Buff1); SetIndexDrawBegin(1,period); SetIndexLabel(1,"BB UP");
   SetIndexStyle(2,DRAW_LINE); SetIndexBuffer(2,Buff2); SetIndexDrawBegin(2,period); SetIndexLabel(2,"BB DOWN");
   SetIndexStyle(3,DRAW_LINE); SetIndexBuffer(3,Buff3); SetIndexDrawBegin(3,period); SetIndexLabel(3,"MA");

   SetIndexStyle(4,DRAW_ARROW,EMPTY,1); SetIndexArrow(4,233); SetIndexBuffer(4,Buff4); SetIndexEmptyValue(4,0.0); SetIndexLabel(4,"signal UP");
   SetIndexStyle(5,DRAW_ARROW,EMPTY,1); SetIndexArrow(5,234); SetIndexBuffer(5,Buff5); SetIndexEmptyValue(5,0.0); SetIndexLabel(5,"signal DOWN");

   SetLevelValue(0,up_level); 
   SetLevelValue(1,dn_level); 

   return(0);
  }
//============================================================================================== The deinitialization function of the indicator ===//
int deinit()
  {

   return(0);
  }
//========================================================================================================== The main function of the indicator ===//
int start()
  {
   double ma, stdev;
      int i, limit, count=IndicatorCounted();

   if(count<0) return(0);
   limit=Bars-count;  
   if(limit>1) limit=Bars-1;

   for(i=limit; i>=0; i--) 
    Buff0[i]=iRSI(NULL,0,rsi_period,rsi_price,i);

   for(i=limit; i>=0; i--) {
    ma=iMAOnArray(Buff0,0,bb_period,0,0,i); //  
    stdev=iStdDevOnArray(Buff0,0,bb_period,0,0,i); // 
    Buff1[i]=ma+bb_dev*stdev; //  
    Buff2[i]=ma-bb_dev*stdev; //  
    } // for

   for(i=limit; i>=0; i--) 
    Buff3[i]=iMAOnArray(Buff0,0,ma_period,0,ma_method,i); 

   if(use_arrows) {
    if(limit<Bars-1) limit++;

    for(i=limit; i>0; i--) {
     if((Buff0[i]<dn_level && Buff0[i]>Buff2[i] && Buff0[i]>Buff3[i]) && (Buff0[i+1]<=Buff2[i+1] || Buff0[i+1]<=Buff3[i+1]))
      Buff4[i]=MathMin(Buff2[i],Buff3[i]);
     if((Buff0[i]>up_level && Buff0[i]<Buff1[i] && Buff0[i]<Buff3[i]) && (Buff0[i+1]>=Buff1[i+1] || Buff0[i+1]>=Buff3[i+1]))
      Buff5[i]=MathMax(Buff1[i],Buff3[i]);
     } // for
    }

   return(0);
  }
//=================================================================================================================================================//

